// components/pusher.js
const _windowWidth = wx.getSystemInfoSync().windowWidth;

Component({
  /**
   * 组件的属性列表
   */
  properties: {
    //  组件显示区域的宽度
    width: {
      type: Number,
      value: _windowWidth
    },
    //  组件显示区域的高度
    height: {
      type: Number,
      value: 0,
    },
    pubID: {
      type: String,
      value: ''
    },
    pushURL: {
      type: String,
      value: ''
    },
    coverImg: {
      type: String,
      value: ''
    },
    enableAudio: {
      type: Boolean,
      value: true
    },
    enableVideo: {
      type: Boolean,
      value: true
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    pusherContext: null,
    userList: [],

    ERROR_OPEN_CAMERA: -4,
    ERROR_OPEN_MIC: -5,
    ERROR_PUSH_DISCONNECT: -6,
    ERROR_CAMERA_MIC_PERMISSION: -7,
    ERROR_EXCEEDS_THE_MAX_MEMBER: -8,
    ERROR_REQUEST_ROOM_SIG: -9,
    ERROR_JOIN_ROOM: -10,
    ERROR_NETWORK_CHANGE: -11,
  },

  /**
   * 组件的方法列表
   */
  methods: {
    //监听推流状态
    onPush(e) {
      let that = this;
      console.log('onPush', e);
      console.log('============== onPush e pubID', this.data.pubID);
      if (!that.data.pusherContext) {
        that.data.pusherContext = wx.createLivePusherContext('rtcpusher');
      }
      var code;
      if (e.detail) {
        code = e.detail.code
      } else {
        code = e
      }
      console.log('>>>>>>>>>>>> 推流情况：', code)
      switch (code) {
        case 1002:
          {
            console.log('推流成功')
            break
          }
        case -1301:
          {
            console.error('打开摄像头失败: ', code)
            that.postEvent('PushError', that.data.ERROR_OPEN_CAMERA, '打开摄像头失败')
            that.stop()
            break
          }
        case -1302:
          {
            console.error('打开麦克风失败: ', code);
            that.postEvent('PushError', that.data.ERROR_OPEN_MIC, '打开麦克风失败');
            that.stop()
            break
          }
        case -1307:
          {
            console.error('推流连接断开: ', code)
            that.postEvent('PushError', that.data.ERROR_PUSH_DISCONNECT, '推流连接断开')
            that.stop();
            break
          }
        case 5000:
          {
            console.log('收到5000: ', code)
            // 收到5000就退房
            that.stop()
            break
          }
        case 1018:
          {
            console.log('进房成功', code)
            break
          }
        case 1019:
          {
            console.log('退出房间', code)
            that.postEvent('PushError', that.data.ERROR_JOIN_ROOM, '加入房间异常，请重试')
            break
          }
        case 1020:
          {
            console.log('成员列表', code, e.detail.message)
            that.onWebRTCUserListPush(e.detail.message)
            break
          }
        case 1021:
          {
            console.log('网络类型发生变化，需要重新进房', code)
            //先退出房间
            that.stop()
            that.postEvent('PushError', that.data.ERROR_NETWORK_CHANGE, '网络类型发生变化，需要重新进房')
            break
          }
        case 2007:
          {
            console.log('视频播放loading: ', e.detail.code)
            break
          };
        case 2004:
          {
            console.log('视频播放开始: ', e.detail.code)
            break
          };
        default:
          {
            console.log('推流情况：', code)
          }
      }
    },

    onPushError (e) {
      10001 == e.detail.errCode && (e.detail.errMsg = '未获取到摄像头功能权限，请删除小程序后重新打开')
      10002 == e.detail.errCode && (e.detail.errMsg = '未获取到录音功能权限，请删除小程序后重新打开')

      this.postEvent('PushError', this.data.ERROR_CAMERA_MIC_PERMISSION, e.detail.errMsg || '未获取到摄像头、录音功能权限，请删除小程序后重新打开')
    },

    postEvent(t, c, d) {
      this.triggerEvent('RoomEvent', {
        tag: t,
        code: c,
        detail: d
      }, {});
    },

    //监听人员变化
    onWebRTCUserListPush(msg) {
      let that = this;

      // console.log('================= onWebRTCUserListPush method', msg);
      if (!msg) {
        return;
      }

      var jsonDic = JSON.parse(msg);
      if (!jsonDic) {
        return;
      }

      // console.log("onWebRTCUserListPush.jsonDict:", jsonDic);
      var newUserList = jsonDic.userlist;
      // console.log('=== newUserList: ', JSON.stringify(newUserList));

      if (!newUserList) {
        return;
      }

      let userList = that.data.userList;
      let joinList = [];
      let leaveList = [];

      userList.map((item, index) => {
        let hasUser = newUserList.find(i => {
          return i.userid === item.pubID
        });

        if (!hasUser) {
          // console.log('删除的', item, index);
          userList.splice(index, 1);
          leaveList.push(item);
        }
      });
      that.postEvent('MemberLeave', 0, leaveList);

      newUserList.map((item, index) => {
        let hasUser = userList.find(i => {
          return i.pubID === item.userid
        });

        if (!hasUser) {
          // console.log('新增的', item);
          userList.push({
            pubID: item.userid,
            playURL: item.playurl,
          });
          joinList.push({
            pubID: item.userid,
            playURL: item.playurl,
          });
        }
      });

      that.postEvent('MemberJoin', 0, joinList);

      that.setData({
        userList: userList
      });
    },

    // 暂停推流
    pause: function () {
      let that = this;

      that.data.pusherContext || (that.data.pusherContext = wx.createLivePusherContext("rtcpusher"));
      that.data.pusherContext && that.data.pusherContext.pause();
    },

    // 停止
    stop() {
      let that = this;

      that.data.pusherContext || (that.data.pusherContext = wx.createLivePusherContext("rtcpusher"));
      that.data.pusherContext && that.data.pusherContext.stop && that.data.pusherContext.stop();
    },

    // 恢复推流
    resume: function () {
      let that = this;

      that.data.pusherContext || (that.data.pusherContext = wx.createLivePusherContext("rtcpusher"));
      that.data.pusherContext && that.data.pusherContext.resume();
    },
    //切换摄像头
    switchCamera: function () {
      let that = this;

      that.data.pusherContext || (that.data.pusherContext = wx.createLivePusherContext("rtcpusher")),
        that.data.pusherContext && that.data.pusherContext.switchCamera({});
    },
   
  }
})
